----------------------------------------------------------------------------------
-- Wb_GPO.vhd
--
-- Copyright (C) 2010 Mario Mauerer
-- 
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or (at
-- your option) any later version.
--
-- This program is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-- General Public License for more details.
--
-- You should have received a copy of the GNU General Public License along
-- with this program; if not, write to the Free Software Foundation, Inc.,
-- 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
--
----------------------------------------------------------------------------------
--This provides some simple General-Purpose-Outputs on the Wishbone-IO-Bus of the AVR-SoC.
--Up to 8 Output vectors can be addressed. 
--The output vectors can be read back (e.g. "Blabla = Blabla | 0x0000) works.

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity Wb_GPO is
	port(
		CLKxCI : in std_logic;
		ResetxSI : in std_logic;
		
		Wb_AckxSO : out std_logic;
		Wb_StbxSI : in std_logic;
		Wb_CycxSI : in std_logic;
		Wb_DatainxDI : in std_logic_vector(7 downto 0);
		Wb_DataoutxDO : out std_logic_vector(7 downto 0);
		Wb_AdrxDI : in std_logic_vector(5 downto 0);
		Wb_WexSI : in std_logic;
		
		GPO_0 : out std_logic_vector(7 downto 0); --General Purpose Output 0
		GPO_1 : out std_logic_vector(7 downto 0) --General Purpose Output 1
		
		--GPO_2 : out std_logic_vector(7 downto 0);
		--GPO_3 : out std_logic_vector(7 downto 0);
		--GPO_4 : out std_logic_vector(7 downto 0);
		--GPO_5 : out std_logic_vector(7 downto 0);
		--GPO_6 : out std_logic_vector(7 downto 0);
		--GPO_7 : out std_logic_vector(7 downto 0);
	);
end Wb_GPO;

architecture Behavioral of Wb_GPO is

signal GPO_0xDN, GPO_0xDP : std_logic_vector(7 downto 0);
signal GPO_1xDN, GPO_1xDP : std_logic_vector(7 downto 0);

begin

--Generate the Wishbone-Ack-Pulse:
Wb_handler : process(Wb_CycxSI, Wb_WexSI, Wb_StbxSI) is 
begin
	Wb_AckxSO <= Wb_CycxSI and Wb_StbxSI;
end process Wb_handler;

--Assign the Outputs:
Output_handler : process(GPO_0xDP, GPO_1xDP) is
begin
	GPO_0 <= GPO_0xDP;
	GPO_1 <= GPO_1xDP;
end process Output_handler;

--Read the Data from the Wishbone-Bus:
Dataread: process(CLKxCI) is
begin
	if rising_edge(CLKxCI) then
		--if Address = 0x08: (LEDs)
		if Wb_WexSI = '1' and Wb_CycxSI = '1' and Wb_StbxSI = '1' and Wb_AdrxDI = "001000" then
			GPO_0xDN <= Wb_DatainxDI;
		--if Address = 0x09:
		elsif Wb_WexSI = '1' and Wb_CycxSI = '1' and Wb_StbxSI = '1' and Wb_AdrxDI = "001001" then
			GPO_1xDN <= Wb_DatainxDI;
		end if;
	end if;
end process Dataread;

--Return the Data to the Wishbone-Bus(IO-Read:)
Datawrite : process(CLKxCI) is
begin
if rising_edge(CLKxCI) then
		--if Address = 0x08: (LEDs)
		if Wb_WexSI = '0' and Wb_CycxSI = '1' and Wb_StbxSI = '1' and Wb_AdrxDI = "001000" then
			Wb_DataoutxDO <= GPO_0xDP; --return current status of the register
		--if Address = 0x09:
		elsif Wb_WexSI = '0' and Wb_CycxSI = '1' and Wb_StbxSI = '1' and Wb_AdrxDI = "001001" then
			Wb_DataoutxDO <= GPO_1xDP; --return current status of the register
		end if;
	end if;

end process Datawrite;

--Update the FlipFlops:
FF_update: process(CLKxCI) is
begin
	if rising_edge(CLKxCI) then
		if ResetxSI = '1' then
			GPO_0xDP <= (others => '0');
			GPO_1xDP <= (others => '0');
		else
			GPO_0xDP <= GPO_0xDN;
			GPO_1xDP <= GPO_1xDN;
		end if;
	end if;
end process FF_update;

end Behavioral;



